@@ -105,7 +105,7 @@ pub struct FuncId<'f> {
 impl<'f> FuncId<'f> {
 	/// # Parameters
 	/// name: fully qualified C++ function name (e.g. cv::Mat::create)
-	/// args: C++ argument names ("unnamed", "unnamed_1", ... for unnamed ones)
+	/// args: C++ argument names ("unnamed" for unnamed ones)
 	pub fn new<const ARGS: usize>(name: &'static str, args: [&'static str; ARGS]) -> FuncId<'static> {
 		FuncId {
 			name: name.into(),
@@ -146,6 +146,12 @@ impl<'f> FuncId<'f> {
 	}
 }
 
+impl fmt::Display for FuncId<'_> {
+	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+		write!(f, "{}({})", self.name, self.args.join(", "))
+	}
+}
+
 #[derive(Clone)]
 pub struct Func<'tu, 'ge> {
 	entity: Entity<'tu>,
@@ -337,7 +343,12 @@ impl<'tu, 'ge> Func<'tu, 'ge> {
 	pub fn is_infallible(&self) -> bool {
 		self.as_field_accessor().is_some()
 			|| matches!(self.entity.get_exception_specification(), Some(ExceptionSpecification::BasicNoexcept) | Some(ExceptionSpecification::Unevaluated))
-			|| settings::FORCE_NOEXCEPT.contains(&(self.cpp_fullname().as_ref(), self.clang_arguments().len()))
+			|| settings::FORCE_INFALLIBLE.contains(&self.func_id())
+	}
+
+	pub fn is_unsafe(&self) -> bool {
+		settings::FUNC_UNSAFE.contains(&self.func_id())
+			|| self.arguments().into_iter().any(|a| a.type_ref().is_pass_by_ptr() && !a.is_user_data())
 	}
 
 	pub fn is_default_constructor(&self) -> bool {
@@ -372,7 +383,7 @@ impl<'tu, 'ge> Func<'tu, 'ge> {
 			Kind::Function | Kind::InstanceMethod(..) | Kind::StaticMethod(..)
 			| Kind::ConversionMethod(..) | Kind::GenericInstanceMethod(..) | Kind::GenericFunction
 			| Kind::FunctionOperator(..) | Kind::InstanceOperator(..) => {
-				let mut out = TypeRef::new(self.entity.get_result_type().expect("Can't get return type"), self.gen_env);
+				let mut out = TypeRef::new_ext(self.entity.get_result_type().expect("Can't get return type"), TypeRefTypeHint::PrimitiveRefAsPointer, None, self.gen_env);
 				if let Some(spec) = self.as_specialized() {
 					if out.is_generic() {
 						let spec_type = spec.get(out.base().cpp_full().as_ref())
@@ -381,6 +392,8 @@ impl<'tu, 'ge> Func<'tu, 'ge> {
 							out.set_type_hint(TypeRefTypeHint::Specialized(spec_type));
 						}
 					}
+				} else if let Some(&over) = settings::ARGUMENT_OVERRIDE.get(&self.func_id()).and_then(|x| x.get("return")) {
+					out.set_type_hint(TypeRefTypeHint::ArgOverride(over))
 				}
 				if let Some(type_ref) = out.as_reference() {
 					type_ref
@@ -392,7 +405,9 @@ impl<'tu, 'ge> Func<'tu, 'ge> {
 				if self.type_hint == FunctionTypeHint::FieldSetter {
 					TypeRef::new(self.gen_env.resolve_type("void").expect("Can't resolve void type"), self.gen_env)
 				} else {
-					Field::new(self.entity, self.gen_env).type_ref()
+					let mut out = Field::new(self.entity, self.gen_env).type_ref();
+					out.set_type_hint(TypeRefTypeHint::PrimitiveRefAsPointer);
+					out
 				}
 			}
 		}
@@ -428,15 +443,13 @@ impl<'tu, 'ge> Func<'tu, 'ge> {
 	}
 
 	pub fn arguments(&self) -> Vec<Field<'tu, 'ge>> {
-		let args = self.clang_arguments();
-
 		let empty_spec_hashmap = HashMap::new();
 		let spec = self.as_specialized().unwrap_or_else(|| &empty_spec_hashmap);
 
 		let is_field_setter = self.as_field_setter().is_some();
 		let arg_overrides = settings::ARGUMENT_OVERRIDE.get(&self.func_id());
 
-		args.into_iter()
+		self.clang_arguments().into_iter()
 			.map(|a| {
 				if is_field_setter {
 					return Field::new_ext(a, FieldTypeHint::FieldSetter, self.gen_env)
@@ -510,7 +523,11 @@ impl<'tu, 'ge> Func<'tu, 'ge> {
 	}
 
 	pub fn func_id(&self) -> FuncId {
-		FuncId::from_entity(self.entity)
+		let mut out = FuncId::from_entity(self.entity);
+		if self.as_field_setter().is_some() {
+			out.args.push("val".into());
+		}
+		out
 	}
 }
 
