@@ -42,6 +42,7 @@ use crate::{
 	Typedef,
 	Vector,
 };
+use crate::settings::ArgOverride;
 
 mod renderer;
 
@@ -53,11 +54,18 @@ pub enum DependentTypeMode {
 
 #[derive(Clone, Copy, Debug, PartialEq)]
 pub enum StrType {
-	StdString,
-	CvString,
+	StdString(StrEnc),
+	CvString(StrEnc),
 	CharPtr,
 }
 
+#[derive(Clone, Copy, Debug, PartialEq)]
+pub enum StrEnc {
+	Text,
+	/// string with binary data, e.g. can contain 0 byte
+	Binary,
+}
+
 #[derive(Clone, Copy, Debug, PartialEq)]
 pub enum Dir<T> {
 	In(T),
@@ -85,9 +93,8 @@ pub enum Kind<'tu, 'ge> {
 #[derive(Clone, Copy, Debug)]
 pub enum TypeRefTypeHint<'tu> {
 	None,
-	Nullable,
-	Slice,
-	NullableSlice,
+	ArgOverride(ArgOverride),
+	PrimitiveRefAsPointer,
 	Specialized(Type<'tu>),
 }
 
@@ -160,7 +167,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 				let pointee_typeref = TypeRef::new_ext(pointee, self.type_hint, self.parent_entity, self.gen_env);
 				if pointee_typeref.as_function().is_some() {
 					pointee_typeref.kind()
-				} else if matches!(self.type_hint, TypeRefTypeHint::Slice | TypeRefTypeHint::NullableSlice) {
+				} else if matches!(self.type_hint, TypeRefTypeHint::ArgOverride(ArgOverride::Slice) | TypeRefTypeHint::ArgOverride(ArgOverride::NullableSlice)) {
 					Kind::Array(TypeRef::new_ext(pointee, self.type_hint, self.parent_entity, self.gen_env), None)
 				} else {
 					Kind::Pointer(TypeRef::new_ext(pointee, self.type_hint, self.parent_entity, self.gen_env))
@@ -499,16 +506,24 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 	}
 
 	pub fn as_string(&self) -> Option<Dir<StrType>> {
-		fn class_string_type(cls: Class) -> Option<StrType> {
+		let class_string_type = |cls: Class| -> Option<StrType> {
 			let cpp_fullname = cls.cpp_fullname();
 			if cpp_fullname.starts_with("std::") && cpp_fullname.ends_with("::string") {
-				Some(StrType::StdString)
+				Some(StrType::StdString(if matches!(self.type_hint, TypeRefTypeHint::ArgOverride(ArgOverride::StringAsBytes)) {
+					StrEnc::Binary
+				} else {
+					StrEnc::Text
+				}))
 			} else if cpp_fullname == "cv::String" {
-				Some(StrType::CvString)
+				Some(StrType::CvString(if matches!(self.type_hint, TypeRefTypeHint::ArgOverride(ArgOverride::StringAsBytes)) {
+					StrEnc::Binary
+				} else {
+					StrEnc::Text
+				}))
 			} else {
 				None
 			}
-		}
+		};
 
 		match self.canonical().kind() {
 			Kind::Class(cls) => {
@@ -555,11 +570,11 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 	}
 
 	pub fn is_std_string(&self) -> bool {
-		matches!(self.as_string(), Some(Dir::In(StrType::StdString)))
+		matches!(self.as_string(), Some(Dir::In(StrType::StdString(_))))
 	}
 
 	pub fn is_cv_string(&self) -> bool {
-		matches!(self.as_string(), Some(Dir::In(StrType::CvString)))
+		matches!(self.as_string(), Some(Dir::In(StrType::CvString(_))))
 	}
 
 	pub fn is_char_ptr_string(&self) -> bool {
@@ -632,6 +647,10 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 		matches!(self.canonical().kind(), Kind::Primitive("bool", _))
 	}
 
+	pub fn is_void_ptr(&self) -> bool {
+		self.as_pointer().map_or(false, |inner| inner.is_void())
+	}
+
 	pub fn as_pointer(&self) -> Option<TypeRef<'tu, 'ge>> {
 		if let Kind::Pointer(out) = self.canonical().kind() {
 			Some(out)
@@ -678,7 +697,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 	}
 
 	pub fn is_nullable(&self) -> bool {
-		matches!(self.type_hint, TypeRefTypeHint::NullableSlice | TypeRefTypeHint::Nullable)
+		matches!(self.type_hint, TypeRefTypeHint::ArgOverride(ArgOverride::NullableSlice) | TypeRefTypeHint::ArgOverride(ArgOverride::Nullable))
 	}
 
 	pub fn as_class(&self) -> Option<Class<'tu, 'ge>> {
@@ -795,6 +814,20 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 		settings::DATA_TYPES.contains(self.cpp_full().as_ref())
 	}
 
+	pub fn can_pass_by_ptr(&self) -> bool {
+		if let Some(inner) = self.as_pointer() {
+			if inner.is_primitive() && !self.as_string().is_some() {
+				return true;
+			}
+		}
+		false
+	}
+
+	pub fn is_pass_by_ptr(&self) -> bool {
+		self.is_void_ptr() ||
+			matches!(self.type_hint, TypeRefTypeHint::PrimitiveRefAsPointer) && self.can_pass_by_ptr()
+	}
+
 	pub fn template_specialization_args(&self) -> Vec<TemplateArg<'tu, 'ge>> {
 		match self.type_ref.get_kind() {
 			TypeKind::Typedef => {
@@ -900,7 +933,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 	}
 
 	pub fn rust_name(&self, name_style: NameStyle, lifetime: Lifetime) -> Cow<str> {
-		self.render(RustRenderer::new(name_style, lifetime))
+		self.render(RustRenderer::new(name_style, lifetime, self.is_pass_by_ptr()))
 	}
 
 	pub fn rust_local(&self) -> Cow<str> {
@@ -944,11 +977,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 		Lifetime::explicit().into_iter().take(self.rust_lifetime_count())
 	}
 
-	pub fn rust_extern(&self) -> Cow<str> {
-		self.rust_extern_with_const(ConstnessOverride::No)
-	}
-
-	pub fn rust_extern_with_const(&self, constness: ConstnessOverride) -> Cow<str> {
+	pub fn rust_extern(&self, constness: ConstnessOverride) -> Cow<str> {
 		let constness = constness.with(self.constness());
 		#[allow(clippy::never_loop)] // fixme use named block when stable
 		'typ: loop {
@@ -981,7 +1010,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 				} else if self.as_string().is_some() {
 					out += "c_char";
 				} else {
-					out += &inner.rust_extern_with_const(ConstnessOverride::Yes(constness))
+					out += inner.rust_extern(ConstnessOverride::Yes(constness)).as_ref()
 				}
 				break 'typ out.into();
 			}
@@ -989,7 +1018,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 				break 'typ format!(
 					"*{cnst}[{typ}; {len}]",
 					cnst=self.constness().rust_qual(true),
-					typ=elem.rust_extern_with_const(ConstnessOverride::Yes(constness)),
+					typ=elem.rust_extern(ConstnessOverride::Yes(constness)),
 					len=len,
 				).into();
 			}
@@ -999,7 +1028,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 					// argv is treated as array of output arguments and it doesn't seem to be meant this way
 					format!("*{cnst}c_char", cnst=elem.clang_constness().rust_qual(true)).into()
 				} else {
-					elem.rust_extern_with_const(ConstnessOverride::Yes(constness))
+					elem.rust_extern(ConstnessOverride::Yes(constness))
 				};
 				break 'typ format!(
 					"*{cnst}{typ}",
@@ -1031,8 +1060,10 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 		let typ = 'decl_type: loop {
 			if let Some(dir) = self.as_string() {
 				break 'decl_type match dir {
-					Dir::In(_) => "&str".into(),
-					Dir::Out(_) => "&mut String".into(),
+					Dir::In(StrType::StdString(StrEnc::Text) | StrType::CvString(StrEnc::Text) | StrType::CharPtr) => "&str".into(),
+					Dir::In(StrType::StdString(StrEnc::Binary) | StrType::CvString(StrEnc::Binary)) => "&[u8]".into(),
+					Dir::Out(StrType::StdString(StrEnc::Text) | StrType::CvString(StrEnc::Text) | StrType::CharPtr) => "&mut String".into(),
+					Dir::Out(StrType::StdString(StrEnc::Binary) | StrType::CvString(StrEnc::Binary)) => "&mut Vec<u8>".into(),
 				};
 			} else if self.is_input_array() {
 				break 'decl_type "&dyn core::ToInputArray".into();
@@ -1069,20 +1100,26 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 		}
 	}
 
-	pub fn rust_return_map(&self, is_safe_context: bool, is_static_func: bool) -> Cow<str> {
+	pub fn rust_return_map(&self, is_safe_context: bool, is_static_func: bool, is_infallible: bool) -> Cow<str> {
 		let unsafety_call = if is_safe_context { "unsafe " } else { "" };
 		if self.as_string().is_some() || self.is_by_ptr() {
 			format!(
-				".map(|r| {unsafety_call}{{ {typ}::opencv_from_extern(r) }} )",
+				"let ret = {unsafety_call}{{ {typ}::opencv_from_extern(ret) }};",
 				unsafety_call=unsafety_call,
 				typ=self.rust_return_func_decl(FishStyle::Turbo, is_static_func),
 			).into()
-		} else if self.as_pointer().map_or(false, |i| !i.is_void()) || self.as_fixed_array().is_some() {
+		} else if self.as_pointer().map_or(false, |i| !i.is_void()) && !self.is_pass_by_ptr() || self.as_fixed_array().is_some() {
 			let ptr_call = if self.constness().is_const() { "ref" } else { "mut" };
+			let error_handling = if is_infallible {
+				".expect(\"Function returned null pointer\")"
+			} else {
+				".ok_or_else(|| Error::new(core::StsNullPtr, \"Function returned null pointer\".to_string()))?"
+			};
 			format!(
-				".and_then(|x| {unsafety_call}{{ x.as_{ptr_call}() }}.ok_or_else(|| Error::new(core::StsNullPtr, \"Function returned Null pointer\".to_string())))",
+				"let ret = {unsafety_call}{{ ret.as_{ptr_call}() }}{error_handling};",
 				unsafety_call=unsafety_call,
 				ptr_call=ptr_call,
+				error_handling=error_handling,
 			).into()
 		} else {
 			"".into()
@@ -1136,10 +1173,10 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 					"callback_arg!({name}_trampoline({tramp_args}) -> {tramp_ret} => {tramp_userdata_arg} in callbacks => {name}({fw_args}) -> {fw_ret})",
 					name=name,
 					tramp_args=tramp_args,
-					tramp_ret=ret.rust_extern(),
+					tramp_ret=ret.rust_extern(ConstnessOverride::No),
 					tramp_userdata_arg=userdata_name,
 					fw_args=fw_args,
-					fw_ret=ret.rust_extern(),
+					fw_ret=ret.rust_extern(ConstnessOverride::No),
 				);
 			}
 		}
@@ -1241,7 +1278,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 	}
 
 	pub fn rust_extern_arg_func_decl(&self, name: &str, constness: ConstnessOverride) -> String {
-		let mut typ = self.rust_extern_with_const(constness);
+		let mut typ = self.rust_extern(constness);
 		if self.as_simple_class().is_some() {
 			*typ.to_mut() = format!("*const {}", typ)
 		}
@@ -1252,7 +1289,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 		if self.as_string().is_some() {
 			"*mut c_void".into()
 		} else {
-			self.rust_extern_with_const(ConstnessOverride::Yes(Constness::Mut))
+			self.rust_extern(ConstnessOverride::Yes(Constness::Mut))
 		}
 	}
 
@@ -1265,10 +1302,16 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 	}
 
 	pub fn rust_arg_post_call(&self, name: &str, _is_function_infallible: bool) -> String {
-		if let Some(Dir::Out(_)) = self.as_string() {
-			format!("string_arg_output_receive!(out, {name}_via => {name})", name = name)
-		} else {
-			"".to_string()
+		match self.as_string() {
+			Some(Dir::Out(StrType::StdString(StrEnc::Text) | StrType::CvString(StrEnc::Text) | StrType::CharPtr)) => {
+				format!("string_arg_output_receive!({name}_via => {name})", name = name)
+			}
+			Some(Dir::Out(StrType::StdString(StrEnc::Binary) | StrType::CvString(StrEnc::Binary))) => {
+				format!("byte_string_arg_output_receive!({name}_via => {name})", name = name)
+			}
+			_ => {
+				"".to_string()
+			}
 		}
 	}
 
@@ -1303,7 +1346,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 					}
 					// implement workaround for race when type with std::string gets generated first
 					// we only want vector<cv::String> because it's more compatible across OpenCV versions
-					if str_type == StrType::StdString {
+					if matches!(str_type, StrType::StdString(_)) {
 						let tref = TypeRef::new(vec_cv_string, self.gen_env);
 						out.push(DependentType::from_vector(tref.as_vector().expect("Not possible unless something is terribly broken")));
 					} else {
@@ -1348,7 +1391,8 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 					if !self.is_generic() && !self.is_void() {
 						if self.as_string().is_some() {
 							let type_ref = TypeRef::new(
-								self.gen_env.resolve_type(&self.cpp_extern_return()).expect("Can't resolve string cpp_extern_return()"),
+								self.gen_env.resolve_type(self.cpp_extern_return().as_ref())
+									.expect("Can't resolve string cpp_extern_return()"),
 								self.gen_env,
 							);
 							let def_location = match def_location {
@@ -1366,7 +1410,7 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 								self.gen_env,
 							)));
 							if self.as_abstract_class_ptr().is_some() {
-								out.push(DependentType::from_abstract_ref_wrapper(AbstractRefWrapper::new(self.clone(), self.gen_env)))
+								out.push(DependentType::from_abstract_ref_wrapper(AbstractRefWrapper::new(self.clone())))
 							}
 						}
 					}
@@ -1447,18 +1491,19 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 
 	pub fn cpp_arg_pre_call(&self, name: &str) -> String {
 		match self.as_string() {
-			Some(Dir::Out(StrType::StdString)) => {
-				return format!("std::string {name}_out", name=name);
+			Some(Dir::Out(StrType::StdString(_))) => {
+				format!("std::string {name}_out", name=name)
 			}
-			Some(Dir::Out(StrType::CvString)) => {
-				return format!("cv::String {name}_out", name=name);
+			Some(Dir::Out(StrType::CvString(_))) => {
+				format!("cv::String {name}_out", name=name)
 			}
 			Some(Dir::Out(StrType::CharPtr)) => {
-				return format!("char* {name}_out = new char[1024]()", name=name);
+				format!("char* {name}_out = new char[1024]()", name=name)
+			}
+			Some(Dir::In(_)) | None => {
+				"".to_string()
 			}
-			Some(_) | None => {}
 		}
-		"".to_string()
 	}
 
 	pub fn cpp_arg_func_call<'a>(&self, name: impl Into<Cow<'a, str>>) -> Cow<'a, str> {
@@ -1469,10 +1514,10 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 				let ptr = if str_type != StrType::CharPtr && self.as_pointer().is_some() { "&" } else { "" };
 				return format!("{ptr}{name}_out", ptr=ptr, name=name).into();
 			}
-			Some(Dir::In(StrType::StdString)) => {
+			Some(Dir::In(StrType::StdString(_))) => {
 				return format!("std::string({name})", name=name).into();
 			},
-			Some(Dir::In(StrType::CvString)) => {
+			Some(Dir::In(StrType::CvString(_))) => {
 				return format!("cv::String({name})", name=name).into();
 			},
 			Some(Dir::In(StrType::CharPtr)) | None => {}
@@ -1504,15 +1549,22 @@ impl<'tu, 'ge> TypeRef<'tu, 'ge> {
 
 	pub fn cpp_arg_post_call(&self, name: &str) -> String {
 		match self.as_string() {
-			Some(Dir::Out(StrType::StdString)) | Some(Dir::Out(StrType::CvString)) => {
-				return format!("*{name} = ocvrs_create_string({name}_out.c_str())", name=name);
+			Some(Dir::Out(StrType::StdString(StrEnc::Text) | StrType::CvString(StrEnc::Text))) => {
+				format!("*{name} = ocvrs_create_string({name}_out.c_str())", name=name)
 			}
 			Some(Dir::Out(StrType::CharPtr)) => {
-				return format!("*{name} = ocvrs_create_string({name}_out)", name=name);
+				format!("*{name} = ocvrs_create_string({name}_out)", name=name)
+			}
+			Some(Dir::Out(StrType::StdString(StrEnc::Binary))) => {
+				format!("*{name} = ocvrs_create_byte_string({name}_out.data(), {name}_out.size())", name=name)
+			}
+			Some(Dir::Out(StrType::CvString(StrEnc::Binary))) => {
+				format!("*{name} = ocvrs_create_byte_string({name}_out.begin(), {name}_out.size())", name=name)
+			}
+			Some(Dir::In(_)) | None => {
+				"".to_string()
 			}
-			_ => {}
 		}
-		"".to_string()
 	}
 
 	// we need cleanup as a separate step from post_call because in cv_ocl_convertTypeStr_int_int_int_charX the
@@ -1562,15 +1614,21 @@ impl fmt::Debug for TypeRef<'_, '_> {
 				},
 			};
 			match str_type {
-				StrType::StdString => {
+				StrType::StdString(StrEnc::Text) => {
 					props.push("std_string");
 				},
-				StrType::CvString => {
+				StrType::CvString(StrEnc::Text) => {
 					props.push("cv_string");
 				},
 				StrType::CharPtr => {
 					props.push("char_ptr_string");
 				},
+				StrType::StdString(StrEnc::Binary) => {
+					props.push("byte_std_string");
+				},
+				StrType::CvString(StrEnc::Binary) => {
+					props.push("byte_cv_string");
+				},
 			}
 		}
 		if self.is_void() {
@@ -1596,7 +1654,7 @@ impl fmt::Debug for TypeRef<'_, '_> {
 		dbg
 			.field("rust_safe_id", &self.rust_safe_id())
 			.field("rust_full", &self.rust_full())
-			.field("rust_extern", &self.rust_extern())
+			.field("rust_extern", &self.rust_extern(ConstnessOverride::No))
 			.field("cpp_safe_id", &self.cpp_safe_id())
 			.field("cpp_full", &self.cpp_full())
 			.field("cpp_extern", &self.cpp_extern())
