@@ -3,7 +3,7 @@ use std::{
 	fmt::{self, Write},
 };
 
-use crate::{Element, settings, StringExt};
+use crate::{Element, settings, StringExt, type_ref::{Dir, StrEnc, StrType}};
 
 use super::{Kind, TemplateArg, TypeRef};
 
@@ -320,11 +320,12 @@ fn render_rust_tpl_decl<'a>(renderer: impl TypeRefRenderer<'a>, type_ref: &TypeR
 pub struct RustRenderer {
 	pub name_style: NameStyle,
 	pub lifetime: Lifetime,
+	pub primitive_ref_as_ptr: bool,
 }
 
 impl RustRenderer {
-	pub fn new(name_style: NameStyle, lifetime: Lifetime) -> Self {
-		Self { name_style, lifetime }
+	pub fn new(name_style: NameStyle, lifetime: Lifetime, primitive_ref_as_ptr: bool) -> Self {
+		Self { name_style, lifetime, primitive_ref_as_ptr }
 	}
 
 	fn wrap_nullable<'a>(&self, type_ref: &TypeRef, typ: Cow<'a, str>) -> Cow<'a, str> {
@@ -340,9 +341,15 @@ impl TypeRefRenderer<'_> for RustRenderer {
 	type Recursed = Self;
 
 	fn render<'t>(self, type_ref: &'t TypeRef) -> Cow<'t, str> {
-		if type_ref.as_string().is_some() {
+		if let Some(str_type) = type_ref.as_string() {
 			#[allow(clippy::if_same_then_else)]
-				return if type_ref.constness().is_const() {
+			return if matches!(str_type, Dir::In(StrType::StdString(StrEnc::Binary) | StrType::CvString(StrEnc::Binary)) | Dir::Out(StrType::StdString(StrEnc::Binary) | StrType::CvString(StrEnc::Binary))) {
+				if self.name_style.turbo_fish_style().is_turbo() {
+					"Vec::<u8>"
+				} else {
+					"Vec<u8>"
+				}
+			} else if type_ref.constness().is_const() {
 				"String" // todo implement receiving const str's
 			} else {
 				"String"
@@ -365,21 +372,26 @@ impl TypeRefRenderer<'_> for RustRenderer {
 				//  because some functions can potentially save the pointer to the value, but it will be destroyed after function call
 				inner.render(self.recurse()).into_owned().into()
 			}
-			Kind::Pointer(inner) | Kind::Reference(inner) => {
-				if inner.is_void() {
-					format!(
-						"*{cnst}c_void",
-						cnst=type_ref.constness().rust_qual(true),
-					).into()
+			Kind::Pointer(inner) if self.primitive_ref_as_ptr || inner.is_void() => {
+				let typ = if inner.is_void() {
+					"c_void".into()
 				} else {
-					let typ = format!(
-						"&{lt: <}{cnst}{typ}",
-						cnst=type_ref.constness().rust_qual(false),
-						lt=self.lifetime,
-						typ=inner.render(self.recurse())
-					);
-					self.wrap_nullable(type_ref, typ.into())
-				}
+					inner.render(self.recurse())
+				};
+				format!(
+					"*{cnst}{typ}",
+					cnst=type_ref.constness().rust_qual(true),
+					typ=typ
+				).into()
+			}
+			Kind::Pointer(inner) | Kind::Reference(inner) => {
+				let typ = format!(
+					"&{lt: <}{cnst}{typ}",
+					cnst=type_ref.constness().rust_qual(false),
+					lt=self.lifetime,
+					typ=inner.render(self.recurse())
+				);
+				self.wrap_nullable(type_ref, typ.into())
 			}
 			Kind::SmartPtr(ptr) => {
 				ptr.rust_name(self.name_style).into_owned().into()
@@ -425,6 +437,7 @@ impl TypeRefRenderer<'_> for RustRenderer {
 		Self {
 			name_style: self.name_style,
 			lifetime: self.lifetime.next().expect("Too many lifetimes"),
+			primitive_ref_as_ptr: self.primitive_ref_as_ptr,
 		}
 	}
 }
@@ -460,22 +473,29 @@ impl<'a> TypeRefRenderer<'a> for CppRenderer<'_> {
 					cnst=cnst,
 					typ=cpp.to_string(),
 					name=space_name,
-				).into()
+				)
 			}
 			Kind::Array(inner, size) => {
 				if let Some(size) = size {
-					format!(
-						"{typ}(*{name})[{size}]",
-						typ=inner.render(self.recurse()),
-						name=self.name,
-						size=size,
-					).into()
+					if self.name.is_empty() {
+						format!(
+							"{typ}**",
+							typ=inner.render(self.recurse()),
+						)
+					} else {
+						format!(
+							"{typ}(*{name})[{size}]",
+							typ=inner.render(self.recurse()),
+							name=self.name,
+							size=size,
+						)
+					}
 				} else {
 					format!(
 						"{typ}*{name}",
 						typ=inner.render(self.recurse()),
 						name=space_name,
-					).into()
+					)
 				}
 			}
 			Kind::StdVector(vec) => {
@@ -485,21 +505,21 @@ impl<'a> TypeRefRenderer<'a> for CppRenderer<'_> {
 					vec_type=vec.cpp_name(self.name_style),
 					elem_type=vec.element_type().render(self.recurse()),
 					name=space_name,
-				).into()
+				)
 			}
 			Kind::Reference(inner) if !self.extern_types => {
 				format!(
 					"{typ}&{name}",
 					typ=inner.render(self.recurse()),
 					name=space_const_name,
-				).into()
+				)
 			}
 			Kind::Pointer(inner) | Kind::Reference(inner) => {
 				format!(
 					"{typ}*{name}",
 					typ=inner.render(self.recurse()),
 					name=space_const_name,
-				).into()
+				)
 			}
 			Kind::SmartPtr(ptr) => {
 				format!(
@@ -508,7 +528,7 @@ impl<'a> TypeRefRenderer<'a> for CppRenderer<'_> {
 					ptr_type=ptr.cpp_name(self.name_style),
 					inner_type=ptr.pointee().render(self.recurse()),
 					name=space_name,
-				).into()
+				)
 			}
 			Kind::Class(cls) => {
 				let mut out: String = cls.cpp_name(self.name_style).into_owned();
@@ -520,7 +540,7 @@ impl<'a> TypeRefRenderer<'a> for CppRenderer<'_> {
 					cnst=cnst,
 					typ=out,
 					name=space_name,
-				).into()
+				)
 			}
 			Kind::Enum(enm) => {
 				format!(
@@ -528,7 +548,7 @@ impl<'a> TypeRefRenderer<'a> for CppRenderer<'_> {
 					cnst=cnst,
 					typ=enm.cpp_name(self.name_style),
 					name=space_name,
-				).into()
+				)
 			}
 			Kind::Typedef(tdef) => {
 				let underlying_type = tdef.underlying_type_ref();
@@ -542,7 +562,7 @@ impl<'a> TypeRefRenderer<'a> for CppRenderer<'_> {
 					cnst=cnst,
 					typ=typ,
 					name=space_name,
-				).into()
+				)
 			}
 			Kind::Generic(generic_name) => {
 				format!(
@@ -550,22 +570,23 @@ impl<'a> TypeRefRenderer<'a> for CppRenderer<'_> {
 					cnst=cnst,
 					typ=generic_name,
 					name=space_name,
-				).into()
+				)
 			}
 			Kind::Function(func) => {
 				let mut typ = func.cpp_name(self.name_style);
 				if typ.contains("(*)") {
-					typ.to_mut().replacen_in_place("(*)", 1, &format!("(*{name})", name=self.name));
+					if !self.name.is_empty() {
+						typ.to_mut().replacen_in_place("(*)", 1, &format!("(*{name})", name=self.name));
+					}
 					typ.into_owned().into()
 				} else {
-					format!("{typ}{name}", typ=typ, name=space_name).into()
+					format!("{typ}{name}", typ=typ, name=space_name)
 				}
 			}
 			Kind::Ignored => {
-				format!("<ignored>{name}", name=space_name).into()
+				format!("<ignored>{name}", name=space_name)
 			}
-		}
-		// render_cpp(self, type_ref, name_style, name, extern_types, constness)
+		}.into()
 	}
 
 	fn recurse(&self) -> Self {
